using System;
using System.IO;
using System.Text;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;

namespace Zcu.Gsvd.MeshLS
{
	class ReaderWrl
	{
		public DataSet ReadFile(string fileName)
		{
			StreamReader sr = new StreamReader(fileName);
			int pointCount = 0;
			int triangleCount = 0;
			while (!sr.EndOfStream)
			{
				string line = sr.ReadLine();
				if (line.Contains("point ["))
				{
					line = sr.ReadLine();
					while (!line.Contains("]"))
					{
						pointCount++;
						line = sr.ReadLine();
					}
				}

				if (line.Contains("coordIndex ["))
				{
					line = sr.ReadLine();
					while (!line.Contains("]"))
					{
						triangleCount++;
						line = sr.ReadLine();
					}
				}
			}

			DataSet result = new DataSet(pointCount, triangleCount, 0);

			sr.Close();
			sr = new StreamReader(fileName);
			int pointIndex = 0;
			int triangleIndex = 0;
			while (!sr.EndOfStream)
			{
				string line = sr.ReadLine();
				if (line.Contains("point ["))
				{
					line = sr.ReadLine();
					while (!line.Contains("]"))
					{
						string[] parts = line.Split(new string[] { " " }, StringSplitOptions.RemoveEmptyEntries);
						if (parts.Length < 3)
						{
							throw new MveException(this, "WRL line contains less that three coordinates.");
						}
						double[] vertex = new double[3];
						for (int i = 0;i<3;i++)
						{
							vertex[i] = double.Parse(parts[i], Globals.nfi);
						}
						result.AddVertex(pointIndex, new Zcu.Mve.Visualization.Point3D(vertex[0], vertex[1], vertex[2]));
						pointIndex++;
						
						line = sr.ReadLine();
					}
				}

				if (line.Contains("coordIndex ["))
				{
					line = sr.ReadLine();
					while (!line.Contains("]"))
					{
						string[] parts = line.Split(new string[] { " " }, StringSplitOptions.RemoveEmptyEntries);
						if (parts.Length < 3)
						{
							throw new MveException(this, "WRL line contains less that three indices.");
						}
						int[] triangle = new int[3];
						for (int i = 0; i < 3; i++)
						{
							triangle[i] = int.Parse(parts[i], Globals.nfi);
						}
						result.AddTriangle(triangleIndex, new Zcu.Mve.Visualization.Triangle(triangle[0], triangle[1], triangle[2]));
						triangleIndex++;

						line = sr.ReadLine();						
					}
				}
			}
			return (result);
		}
	}
}
